<?php

namespace EventPortal\Http\Controllers\Admin\Speaker;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\User\Models\User;
use EventPortal\Domain\Util\AdminRender;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Inertia\Response;

class SpeakerController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:speaker.show')->only('show');
        $this->middleware('check-permission:speaker.store')->only(['create', 'store']);
        $this->middleware('check-permission:speaker.update')->only(['edit', 'update']);
        $this->middleware('check-permission:speaker.destroy')->only('delete');
    }

    public function index(): Response
    {
        $table = Table::make('speaker.index')
            ->query(Speaker::joinRelationship('user'))
            ->request(request())
            ->get();

        $breadcrumbs = Breadcrumb::make()
            ->add('Speaker', route('admin.speaker.index'))
            ->get();

        $quickButton = Button::make('Create speaker')
            ->method(InertiaMethod::VISIT())
            ->icon('plus')
            ->route('admin.speaker.create');

        return AdminRender::component('Admin/Speaker/Index')
            ->key('speaker.index')
            ->withTable($table)
            ->withBreadcrumbs($breadcrumbs)
            ->withQuickButton($quickButton, has_permission('speaker.store'))
            ->get();
    }

    public function create(): Response
    {
        $form = Form::make('speaker.create')
            ->model(new User)
            ->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Speakers', route('admin.speaker.index'))
            ->add('Create a speaker')
            ->get();

        return AdminRender::component('Admin/Speaker/Create')
            ->key('speaker.create')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->get();
    }

    public function store(Request $request): RedirectResponse
    {
        $payload = Form::make('speaker.create')
            ->model(new User)
            ->validate($request->all())
            ->handle($request->all());

        toast()->success(trans('speaker.created'));

        return redirect()->route('admin.speaker.show', $payload->getContext('speaker'));
    }

    public function edit(Speaker $speaker): Response
    {
        $form = Form::make('speaker.edit')
            ->model($speaker)
            ->context($speaker->user)
            ->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Speakers', route('admin.speaker.index'))
            ->add($speaker->name, route('admin.speaker.show', $speaker))
            ->add('Edit speaker')
            ->get();

        return AdminRender::component('Admin/Speaker/Edit')
            ->key('speaker.edit')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($speaker->name)
            ->get();
    }

    public function update(Request $request, Speaker $speaker): RedirectResponse
    {
        Form::make('speaker.edit')
            ->context($speaker->user)
            ->model($speaker)
            ->validate($request->all())
            ->handle($request->all());

        $speaker
            ->user()
            ->update(Arr::only($request->all(), array_keys($speaker->user->getAttributes())));

        toast()->success(trans('speaker.updated'));

        return redirect()->route('admin.speaker.show', $speaker);
    }

    public function show(Speaker $speaker): Response
    {
        $form = Form::make('speaker.show')->model($speaker)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Speakers', route('admin.speaker.index'))
            ->add($speaker->name)
            ->get();

        return AdminRender::component('Admin/Speaker/Show')
            ->key('speaker.show')
            ->withData($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($speaker->name)
            ->get();
    }
}
