<?php

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\ButtonDropdown;
use EventPortal\Domain\Component\Column\DateColumn;
use EventPortal\Domain\Component\Column\StatsColumn;
use EventPortal\Domain\Component\Column\TextColumn;
use EventPortal\Domain\Component\Filter\DateFilter;
use EventPortal\Domain\Component\Filter\DropdownFilter;
use EventPortal\Domain\Component\Filter\StatusFilter;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Statistics\Stat;
use EventPortal\Domain\Util\Status;
use EventPortal\Enums\InertiaMethod;
use Illuminate\Database\Eloquent\Builder;

Table::make('event.index')
    ->setFilters(function () {
        return [
            DateFilter::make('Filter by older than')
                ->forColumn('date')
                ->name('filter_by_older_than')
                ->lookup(function (Builder $query, $value) {
                    return $query->hasDateBeforeOrEqualTo($value);
                }),
            DateFilter::make('Filter by newer than')
                ->forColumn('date')
                ->name('filter_by_newer_than')
                ->lookup(function (Builder $query, $value) {
                    return $query->hasDateAfterOrEqualTo($value);
                }),
            DropdownFilter::make('Filter by event type')
                ->forColumn('type')
                ->options([
                    Option::make('Congress'),
                    Option::make('Seminar'),
                ]),
            StatusFilter::make('Filter by event status')
                ->forColumn('status')
                ->options(Status::make('event')->forFilters()),
        ];
    })
    ->setColumns(function () {
        return [
            TextColumn::make('Event name')->name('name')->searchable()->default(),
            TextColumn::make('Event type')->name('type'),
            DateColumn::make('Event date')
                ->name('date')
                ->value(function (Event $event) {
                    return $event->readable_dates;
                })
                ->default(),
            StatsColumn::make('Delegate stats')
                ->calculated()
                ->value(static function (Event $event): array {
                    $event = $event->withCount([
                        'delegates as delegates_invited' => function (Builder $query): void {
                            $query->where('status', 'invited');
                        },
                        'delegates as delegates_registered' => function (Builder $query): void {
                            $query->where('status', 'registered');
                        },
                        'delegates as delegates_declined' => function (Builder $query): void {
                            $query->where('status', 'declined');
                        },
                    ])
                    ->first();

                    return [
                        Stat::make('Delegate invited', $event->delegates_invited)->primary(),
                        Stat::make('Delegate registered', $event->delegates_registered)->secondary(),
                        Stat::make('Delegate declined', $event->delegates_declined)->tertiary(),
                    ];
                })
                ->name('stats')
                ->default(),
            TextColumn::make('Event status')->name('status')->default(),
        ];
    })
    ->setRowAction(function (Event $event) {
        return ButtonDropdown::make('Manage event')
            ->actions([
                Button::make('View')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.show', ['event' => $event->id]),
                Button::make('Edit')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.edit', ['event' => $event->id]),
            ]);
    });
